

-- USER CONFIG AREA -----------------------------------------------------------

console = true -- true/false: display debug messages in the console

------------------------------------------------------- END OF USER CONFIG AREA

-- UTILITIES -------------------------------------------------------------

function Open_URL(url)
  if not OS then local OS = reaper.GetOS() end
  if OS=="OSX32" or OS=="OSX64" then
    os.execute("start \"\" \"".. url .. "\"")
   else
    os.execute("start ".. url)
  end
end

function CheckSWS()
  if not reaper.BR_SetItemEdges then
    local retval = reaper.ShowMessageBox("SWS extension is required by this script.\nHowever, it doesn't seem to be present for this REAPER installation.\n\nDo you want to download it now ?", "Warning", 1)
    if retval == 1 then
      Open_URL("http://www.sws-extension.org/")
    end
  else
    return true
  end
end



reaper.Main_OnCommand(40421, 0) -- Item: Select all items in track

-- Save item selection
function SaveSelectedItems (table)
  for i = 0, reaper.CountSelectedMediaItems(0)-1 do
    table[i+1] = reaper.GetSelectedMediaItem(0, i)
  end
end


-- Display a message in the console for debugging
function Msg(value)
  if console then
    reaper.ShowConsoleMsg(tostring(value) .. "\n")
  end
end

-- SAVE INITIAL VIEW
function SaveView()
  start_time_view, end_time_view = reaper.BR_GetArrangeView(0)
end


-- RESTORE INITIAL VIEW
function RestoreView()
  reaper.BR_SetArrangeView(0, start_time_view, end_time_view)
end

function MultiSplitMediaItem(item, times)

  local item_pos = reaper.GetMediaItemInfo_Value(item, "D_POSITION")
  local item_end = reaper.GetMediaItemInfo_Value(item, "D_LENGTH") + item_pos

  -- create array then reserve some space in array
  local items = {}

  -- add 'item' to 'items' array
  table.insert(items, item)

  -- for each time in times array do...
  for i, time in ipairs(times) do

    if time > item_end then break end

    if time > item_pos and time < item_end then

      -- store item so we can split it next time around
      item = reaper.SplitMediaItem(item, time)

      -- add resulting item to array
      table.insert(items, item)

    end

  end

  -- return 'items' array
  return items

end

function DeleteItem( item )


  local track = reaper.GetMediaItemTrack( item )
  reaper.DeleteTrackMediaItem( track, item )

end

function CopyMediaItem(item, track, position)

  reaper.SelectAllMediaItems(0 , false)
  reaper.SetMediaItemSelected( item, true)

  reaper.SetEditCurPos(position, false, false)

  reaper.Main_OnCommand( 40698, 0 ) -- Edit: Copy items

  -- Duplicate items track
  reaper.SetOnlyTrackSelected( track )
  reaper.Main_OnCommand( 40914, 0 ) -- Track: Set first selected track as last touched track
  reaper.Main_OnCommand( 40058, 0 ) -- Item: Paste items/tracks

  local new_item = reaper.GetSelectedMediaItem(0, 0)

  return new_item

end

function GetMIDISources()

  local sources = {}

  for i, item in ipairs(init_sel_items) do
    local take = reaper.GetActiveTake( item )
    if take and not reaper.TakeIsMIDI( take ) then
      local source = reaper.GetMediaItemTake_Source( take )
      local filename = reaper.GetMediaSourceFileName( source, "" )
      midi_filename = filename:gsub("%.(.+)", '.mid')
      midi_exists = reaper.file_exists( midi_filename )
      if not midi_exists then
        midi_filename = filename:gsub("%.(.+)", '.midi')
        midi_exists = reaper.file_exists( midi_filename )
      end
      if midi_exists then
        if not sources[midi_filename] then sources[midi_filename] = {} end
        table.insert(sources[midi_filename], item)
      end
    end
  end

  return sources

end
--------------------------------------------------------- END OF UTILITIES

-- Move and glue

function move()
  
  reaper.Main_OnCommand(40001, 0) -- Track: Insert new track
  
  commandID1 = reaper.NamedCommandLookup("_RS8dae584448854e669ae2c22427811e3fe367addf")
  reaper.Main_OnCommand(commandID1, 0) -- Script: ReaTrak move selected items to selected track.lua
  
  reaper.Main_OnCommand(41588, 0) -- Item: Glue items
  
  --commandID3 = reaper.NamedCommandLookup("_RS6f7bce81b208eb305b216b3cc9ba19633cc771db")
  --reaper.Main_OnCommand(commandID3, 0) -- Script: ReaTrak set take name to current reatrak file.lua
    selitem = reaper.GetSelectedMediaItem( 0, 0 )
    
    take = reaper.GetMediaItemTake(selitem, 0)
    
    if take and reaper.TakeIsMIDI( take ) then
    
       takename = reaper.GetTakeName(take)
    
       new_takename = string.match(takename, "- (.*)")
       
       new_takename2 = string.gsub(new_takename, "-glued", ".mid") 
    
       retval, stringNeedBig = reaper.GetSetMediaItemTakeInfo_String(take, "P_NAME", new_takename2, 1)  
    end
  
end



-- Main function
function main()

  new_items = {}

  sources = GetMIDISources()

  track = reaper.GetSelectedTrack(0,0)
  
  

  for midi_filename, items in pairs( sources ) do

    local first_item

    for i, item in ipairs( items ) do
      reaper.SelectAllMediaItems(0, false)

      local take = reaper.GetActiveTake( item )
      local take_name = reaper.GetTakeName( take )
      local source = reaper.GetMediaItemTake_Source( take )

      local item_pos = reaper.GetMediaItemInfo_Value( item, "D_POSITION" )
      local item_len = reaper.GetMediaItemInfo_Value( item, "D_LENGTH" )
      local item_end  = item_pos + item_len
      local take_offset = reaper.GetMediaItemTakeInfo_Value( take, "D_STARTOFFS" )
      local take_playrate = reaper.GetMediaItemTakeInfo_Value( take, "D_PLAYRATE" )

      if i == 1 then
        reaper.SetEditCurPos(0, false, false) -- Should be item pos
        reaper.InsertMedia( midi_filename, 0 )
        local retval, offs, len, rev = reaper.PCM_Source_GetSectionInfo( source )

        local take_off_comp = take_offset * (1/take_playrate)
        local item_source_pos = item_pos - take_off_comp

        local new_item = reaper.GetSelectedMediaItem( 0, 0 )
        table.insert(new_items, new_item)
        local new_take = reaper.GetActiveTake( new_item )

        reaper.BR_SetItemEdges( new_item, item_source_pos + take_offset / take_playrate , item_source_pos + item_len + take_offset / take_playrate )
        reaper.SetMediaItemTakeInfo_Value( new_take, "D_STARTOFFS", take_offset / take_playrate)
        first_item = new_item
      else
        local new_item = CopyMediaItem(first_item, track, item_pos)
        table.insert(new_items, new_item)
        local new_take = reaper.GetActiveTake( new_item )
        reaper.SetMediaItemTakeInfo_Value( new_take, "D_STARTOFFS", take_offset / take_playrate )
        reaper.BR_SetItemEdges( new_item, item_pos, item_end )
        local retval, string = reaper.GetSetMediaItemTakeInfo_String( new_take, "P_NAME", take_name, true )
      end
    end
  end

  for i, item in ipairs( new_items ) do
    reaper.SetMediaItemSelected( item, true )
  end
  
  --move()

end



         

-- INIT

-- See if there is items selected
count_sel_items = reaper.CountSelectedMediaItems(0)
count_sel_tracks = reaper.CountSelectedTracks(0)

sws = CheckSWS()

if not sws then return end

if count_sel_items > 0 and count_sel_tracks > 0 then

  reaper.PreventUIRefresh(1)

  reaper.Undo_BeginBlock() -- Begining of the undo block. Leave it at the top of your main function.

  init_sel_items =  {}
  SaveSelectedItems(init_sel_items)

  reaper.ClearConsole()

  reaper.Main_OnCommand( 40914, 0 ) -- Track: Set first selected track as last touched track

  init_cur_pos = reaper.GetCursorPosition()

  SaveView()

  main()
  
  reaper.SetEditCurPos(init_cur_pos, false, false)
  
  RestoreView()

  reaper.Undo_EndBlock("ReaTrak Midi match same section to audio", -1) -- End of the undo block. Leave it at the bottom of your main function.

  reaper.UpdateArrange()

  reaper.PreventUIRefresh(-1)

end

move()
