--[[
 * ReaScript Name: Import markers and regions from tab-delimited CSV file
 * Description: See title.
 * Instructions: Select a track. Run.
 * Author: X-Raym
 * Author URI: http://www.extremraym.com
 * Repository: GitHub > X-Raym > EEL Scripts for Cockos REAPER
 * Repository URI: https://github.com/X-Raym/REAPER-EEL-Scripts
 * Links
    Forum Thread https://forum.cockos.com/showthread.php?p=1670961
 * Licence: GPL v3
 * REAPER: 5.0
 * Version: 1.0
--]]

--[[
 * Changelog:
 * v1.0 (2019-01-26)
  + Initial Release
--]]

-- USER CONFIG AREA -----------------------------------------------------------
-- Duplicate and Rename the script if you want to modify this.
-- Else, a script update will erase your mods.

-- Display a message in the console for debugging
function Msg(value)
  if console then
    reaper.ShowConsoleMsg(tostring(value) .. "\n")
  end
end

console = true -- true/false: display debug messages in the console
sep = "," -- default sep

--(1)pattern,(2)type,(3)A verse B chorus,(4)weight,(5)mask,(6)duration,(7)bar
--(1)shot,(2)0 shot 1 hold,(3)bar,(4)ticks,(5)duration ticks,weight,volume

col_pos = 7 -- Position column index in the CSV
col_pos_end = 4 -- Length column index in the CS
col_len = 6 -- Length column index in the CSV
col_name = 2 -- Name column index in the CSV
col_color = 3
col_pattern = 1
col_ticks = 5

unit = 4
bpm, beat_per_measure = reaper.GetProjectTimeSignature2(0)

------------------------------------------------------- END OF USER CONFIG AREA

function ColorHexToInt(hex)
  hex = hex:gsub("#", "")
  local R = tonumber("0x"..hex:sub(1,2))
  local G = tonumber("0x"..hex:sub(3,4))
  local B = tonumber("0x"..hex:sub(5,6))
  return reaper.ColorToNative(R, G, B)
end

-- Optimization
local reaper = reaper

-- CSV to Table
-- http://lua-users.org/wiki/LuaCsv
function ParseCSVLine (line,sep)
  local line = string.gsub(line, "{.*}", "") -- remove { } and text between them
  local line = string.gsub(line, ";.*", "") -- remove ; and text after it
  local line = string.gsub(line, "pattern,PreFill.*", "") -- remove pattern,PreFill line
  local res = {}
  local pos = 1 
  sep = sep or ','
  while true do
    local c = string.sub(line,pos,pos)
    if (c == "") then break end
    if (c == '"') then
      -- quoted value (ignore separator within)
      local txt = ""
      repeat
        local startp,endp = string.find(line,'^%b""',pos)
        txt = txt..string.sub(line,startp+1,endp-1)
        pos = endp + 1
        c = string.sub(line,pos,pos)
        if (c == '"') then txt = txt..'"' end
        -- check first char AFTER quoted string, if it is another
        -- quoted string without separator, then append it
        -- this is the way to "escape" the quote char in a quote. example:
        --   value1,"blub""blip""boing",value3  will result in blub"blip"boing  for the middle
      until (c ~= '"')
      table.insert(res,txt)
      assert(c == sep or c == "")
      pos = pos + 3
    else
      -- no quotes used, just look for the first separator
      local startp,endp = string.find(line,sep,pos)
      if (startp) then
        table.insert(res,string.sub(line,pos,startp-1))
        pos = endp + 1
      else
        -- no separator found -> use rest of string and terminate
        table.insert(res,string.sub(line,pos))
        break
      end
    end
  end
  return res
end


-- UTILITIES -------------------------------------------------------------

-- Display a message in the console for debugging
function Msg(value)
  if console then
    reaper.ShowConsoleMsg(tostring(value) .. "\n")
  end
end

function ReverseTable(t)
    local reversedTable = {}
    local itemCount = #t
    for k, v in ipairs(t) do
        reversedTable[itemCount + 1 - k] = v
    end
    return reversedTable
end

--------------------------------------------------------- END OF UTILITIES

function read_lines(filepath)

  lines = {}

  local f = io.input(filepath)
  repeat

    s = f:read ("*l") -- read one line

    if s then  -- if not end of file (EOF)
      table.insert(lines, ParseCSVLine (s,sep))
    end

  until not s  -- until end of file
         

  f:close()

end


-- Main function
function main()

  folder = filetxt:match[[^@?(.*[\/])[^\/]-$]]
  
  for i, line in ipairs( lines ) do
    if line[col_pattern] == "pattern" then
    
    
    
    
   
     if i > 1 then
      
      -- Name Variables
      local pos = line[col_pos]
      --local pos_end = tonumber(line[col_pos] +1) --tonumber(line[col_pos_end])
      local len =  line[col_len] 
      local name = line[col_name]
      local color = 0
      if line[col_name] == "Count-in" then
        color = ColorHexToInt("#FF80C0")|0x1000000
        duration = 2
      end      
      if (line[col_color] == "A" or line[col_color] == "a") and line[col_name] == "PostFill" then
        color = ColorHexToInt("#1127AE")|0x1000000
        duration = 1
      end
      if (line[col_color] == "A" or line[col_color] == "a") and line[col_name] == "Normal" then
        color = ColorHexToInt("#3776EB")|0x1000000
        duration = 1
      end
      if (line[col_color] == "A" or line[col_color] == "a") and line[col_name] == "Fill" then
        color = ColorHexToInt("#71BEF1")|0x1000000
        duration = 1
      end
      if line[col_color] == "Aending" and (line[col_name] == "ending" or line[col_name] == "Ending") then
        color = ColorHexToInt("#97D0FE")|0x1000000
        duration = 2
      end
      if line[col_color] == "0" and (line[col_name] == "ending" or line[col_name] == "Ending") then -- any ending color
        color = ColorHexToInt("#42F4D1")|0x1000000
        duration = 2
        Msg("Any Ending Color ".. color)
      end
      if (line[col_color] == "B" or line[col_color] == "b") and line[col_name] == "PostFill" then
        color = ColorHexToInt("#0B7427")|0x1000000
        duration = 1
      end
      if (line[col_color] == "B" or line[col_color] == "b") and line[col_name] == "Normal" then
        color = ColorHexToInt("#11AE3B")|0x1000000
        duration = 1
      end
      if (line[col_color] == "B" or line[col_color] == "b") and line[col_name] == "Fill" then
        color = ColorHexToInt("#50ED7B")|0x1000000
        duration = 1
      end      
      if line[col_color] == "Bending"  and (line[col_name] == "ending" or line[col_name] == "Ending") then
        color = ColorHexToInt("#9EFEB6")|0x1000000
        duration = 2
      end      


        pos2 = ((((pos+1) * beat_per_measure)/ bpm) * 60)  -- +2 bar offset count-in
        --len2 = (((len * beat_per_measure)/ bpm) * 60)
        len2 = (((duration * beat_per_measure)/ bpm) * 60)
        reaper.AddProjectMarker2( 0, true, pos2,pos2 + len2, "", -1, color ) 
      end
      end
    end
      
end

function main2()

--(1)pattern,(2)type,(3)A verse B chorus,(4)weight,(5)mask,(6)duration,(7)bar
--(1)shot,(2)0 shot 1 hold,(3)bar,(4)ticks,(5)duration ticks,weight,volume

--col_pos = 7 -- Position column index in the CSV
--col_pos_end = 4 -- Length column index in the CS
--col_len = 6 -- Length column index in the CSV
--col_name = 2 -- Name column index in the CSV
--col_color = 3
--col_pattern = 1
--col_ticks = 5

  folder = filetxt:match[[^@?(.*[\/])[^\/]-$]]

  for i, line in ipairs( lines ) do
  
    --bug_pattern = "Shot{ Kick (applicable for Brushes & Sticks)  },1,320,0,725,1,90"
    
    line_pattern = line[col_pattern]

    --shot_pattern = string.gsub(bug_pattern, "{.*}", "")

    

    if line_pattern  == "Shot" then

    

     if i > 1 then
      
      -- Name Variables
      local pos = line[col_color] -- bar is column 2 for Shots
      --local pos_end = tonumber(line[col_pos] +1) --tonumber(line[col_pos_end])
      local len =  line[col_len] 
      local name = line[col_name]
      local color = 0
      local dura_ticks = line[col_ticks]
      local ticks = line[col_pos_end]
      if line[col_name] == "0" then
        color = ColorHexToInt("#BFBFBF")|0x1000000 -- Hold
        
      end
      if line[col_name] == "1" then
        color = ColorHexToInt("#7F7F7F")|0x1000000 -- Shot
        
      end
        --ticks_pos = (dura_ticks / 220)
        --duration = (ticks /220)
        duration2 = (dura_ticks / (bpm *2))
        ticks_pos = (ticks / (bpm *2))
        
 
        --[[
        Msg("===Shots===")
        Msg("BPM ".. bpm)
        Msg("Bar ".. pos+2)
        Msg("Ticks ".. ticks)
        Msg("Ticks pos in ".. ticks_pos)        
        Msg("Druation ticks ".. dura_ticks)
        Msg("Duration ".. duration2)
        --]]
        
        
        
        --pos1 = string.gsub(pos,"{(.*)}","")    
        --Msg("Pos " .. pos)
        pos2 = ((((pos+1) * beat_per_measure)/ bpm) * 60) --+ 2 -- +_2 bar offset count-in
        bar_pos_start = pos2 + ticks_pos
        --len2 = (((len * beat_per_measure)/ bpm) * 60)
        --len2 =  (((duration * beat_per_measure)/ bpm) * 60)
        
        --Msg("Duration ".. duration)
        reaper.AddProjectMarker2( 0, true, bar_pos_start, bar_pos_start + duration2, "" , -1, color ) 
      end
      end
    end
      
end


function snap_all_regions_to_grid_skip_shots()

      reaper.Main_OnCommand(40754, 0) -- Enable snap
      
      region_count , num_markersOut, num_regionsOut = reaper.CountProjectMarkers(0)

      for i=0, region_count -1 do
      
       --EnumProjectMarkers(i, is_region, region_start, region_end, #name, region_id)
       retval, isrgnOut, posOut, rgnendOut, region_name, markrgnindexnumberOut, colorOut = reaper.EnumProjectMarkers3(0, i)      
      
      if colorOut == ColorHexToInt("#BFBFBF")|0x1000000 or colorOut == ColorHexToInt("#7F7F7F")|0x1000000 then drum_pattern = 0
      
      else drum_pattern = 1 
         
         region_snapped_start =  reaper.SnapToGrid(0, posOut)
         region_snapped_end =  reaper.SnapToGrid(0, rgnendOut) 
         
         --SetProjectMarker(region_id, 1, region_snapped_start, region_snapped_end, #name)
         
         reaper.SetProjectMarker3( 0, markrgnindexnumberOut, drum_pattern, region_snapped_start, region_snapped_end, region_name , colorOut )
         
      
      end
    end
end

-- INIT
retval, filetxt = reaper.GetUserFileNameForRead("", "Import markers and regions", "txt")

--filetxt1 ="Mac:/bb/Drums/ModCountry16ths^/ModCountry16ths^01-ClHSSK,OpHSSK/ModCountry16ths^01-ClHSSK,OpHSSK_080_Style.txt"
folder = filetxt:match[[^@?(.*[\/])[^\/]-$]]


os_type = reaper.GetOS() -- "Mac OSX" -- "Win64" --
slash = string.match(os_type, "Win") and "\\" or "/"

if os_type ~= "Win32" and os_type ~= "Win64" then
--Mac OS
   dir1 = string.gsub(filetxt, "/[^/]*$", "")
   dir = directory1 .."/"
   txt_filename1 = string.match(filetxt, "/[^/]*$")
   txt_filename = string.gsub(txt_filename, "/", "")
   dir_back = string.gsub(dir1,"(.*)/.*$","%1")
   dir_back = dir_back .."/"
   open_file = dir ..  blank_ext .. realdrums_extension
   open_file_back = dir1_back .. blank_ext .. realdrums_extension
   --Needs root drum name from txt wavname=*****
else
--Win OS
   dir1 = string.gsub(filetxt, "\\[^\\]*$", "")
   dir = dir1 .."\\"
   txt_filename1 = string.match(filetxt, "\\[^\\]*$")
   txt_filename = string.gsub(txt_filename1, "\\", "")
   blank_ext = string.gsub(txt_filename, ".txt", "")
   dir_back = string.gsub(dir1,"(.*)\\.*$","%1")
   dir_back = dir_back .."\\"
   --open_file = dir .. "ModCountry16ths^_080_Style.wav" --blank_ext .. realdrums_extension
   open_file_back = "ModCountry16ths^_080_Style.wav" --dir1_back .. blank_ext .. realdrums_extension
   
   --use in function to check dir ot dir_back
    --check_path = reaper.file_exists(open_file) 
   
    --check_path = reaper.file_exists(open_file_back) 
   
end



--Msg("Dir ".. directory1)

--Msg("Dir back ".. directory_back)
--Msg("Dir back\\ ".. directory1_back)        
--foo:gsub("(.*)\\.*$","%1")
--Msg("Dir ".. directory)
--Msg("Dir back ".. path_back)
--slash = "/" --string.match(os_type, "Win") and "\\" or "/"

--Msg("Folder ".. directory)
--Msg("Filename1 ".. txt_filename1) 
--Msg("Filename ".. txt_filename) 

--"("..slash..")%s(a%+)"


--[[
--Msg("Path ".. filetxt)
--directory = string.gsub(filetxt, "\\[^\\]*$", "")
directory = string.gsub(filetxt_mac, "("..slash..")[^("..slash..")]*$", "")
--txt_filename1 = string.match(filetxt, "\\[^\\]*$")
--txt_filename1 = string.match(filetxt, "("..slash..")[^("..slash..")]*$")
txt_filename1, txt_filename1 = string.match(filetxt_mac, "\\[^\\]*$"), string.match(filetxt_mac, "\\[^\\]*$")
--txt_filename = string.gsub(txt_filename1, "\\", "")
txt_filename = string.gsub(txt_filename1, "("..slash..")", "")  
--]]

  



if retval then

  reaper.PreventUIRefresh(1)

  reaper.Undo_BeginBlock() -- Begining of the undo block. Leave it at the top of your main function.
  
  --reaper.ClearConsole()

  read_lines(filetxt)
  
  -- reaper.Main_OnCommand( reaper.NamedCommandLookup( "_SWSMARKERLIST10" ), -1) -- SWS: Delete all regions

  main()
  
  main2()
  
  snap_all_regions_to_grid_skip_shots()

  reaper.Undo_EndBlock("ReaTrak biab drums txt to regions", -1) -- End of the undo block. Leave it at the bottom of your main function.

  reaper.UpdateArrange()

  reaper.PreventUIRefresh(-1)

end

::finish::
