-- NoIndex: true

--[[
  Lokasenna_GUI example

  - Demonstration of the Listbox, Menubar, and TextEditor classes

]]--

-- The Core library must be loaded prior to anything else

--[[ ----
local lib_path = reaper.GetExtState("Lokasenna_GUI", "lib_path_v2")
if not lib_path or lib_path == "" then
    reaper.MB("Couldn't load the Lokasenna_GUI library. Please run 'Script: Set Lokasenna_GUI v2 library path.lua' in your Action List.", "Whoops!", 0)
    return
end
loadfile(lib_path .. "Core.lua")()
]]--

-----DEBUG CONTROL--------------------------------------------------------------

debug_output = true -- true/false --display debug messages in the console

-----DISPLAY CONSOLE MESSAGE----------------------------------------------------

-- Display a message in the console for debugging, message can be a number variable 
-- or a string but not mixed. No line feed character is added.
function Msg(value)
  --returns nothing

  if debug_output then --check if flag is set true (at the start of this script)
    reaper.ShowConsoleMsg(tostring(value) )
  end
end


local dm, _ = debug_mode
local function Msg(str)
  reaper.ShowConsoleMsg(tostring(str).."\n")
end

local info = debug.getinfo(1,'S');
script_path = info.source:match[[^@?(.*[\/])[^\/]-$]]


-- I hate working with 'requires', so I've opted to do it this way.
-- This also works much more easily with my Script Compiler.
local function req(file)
  
  if missing_lib then return function () end end
  
  local ret, err = loadfile(script_path .. file)
  if not ret then
    reaper.ShowMessageBox("Couldn't load "..file.."\n\nError: "..tostring(err), "Library error", 0)
    missing_lib = true          
    return function () end

  else 
    return ret
  end     

end

  

-- The Core library must be loaded prior to any classes, or the classes will throw up errors
-- when they look for functions that aren't there.
req("ReaTrak_Core.lua")()

-- For better cross-platform behavior.
local sep = GUI.file_sep

req("ReaTrak_Classes/Class - Menubar.lua")()
req("ReaTrak_Classes/Class - Button.lua")()
req("ReaTrak_Classes/Class - Listbox.lua")()
req("ReaTrak_Classes/Class - TextEditor.lua")()

-- If any of the requested libraries weren't found, abort the script.
if missing_lib then return 0 end



------------------------------------
-------- Menu functions ------------
------------------------------------


local mnu_sections = {
    
    help = function()
        commandID1 = reaper.NamedCommandLookup("_RSbe31a3de2526d47fa8357af06379275cceb291c2")
        reaper.Main_OnCommand(commandID1, 0) -- Script: ReaTrak sections help.lua
    end,
    
    count_in = function()
        commandID1 = reaper.NamedCommandLookup("_b2ce30f0f372ec4995a64701ccb9169a")
        reaper.Main_OnCommand(commandID1, 0) -- Custom: ReaTrak Set to Count-In
    end,
    
    intro = function()
        commandID3 = reaper.NamedCommandLookup("_b6b4a8efc7c17e4293bd5b72e6617705")
        reaper.Main_OnCommand(commandID3, 0) -- Custom: ReaTrak Set to Intro
    end,    

    intro_post_fill = function()
        commandID2 = reaper.NamedCommandLookup("_f7c5b3a543e08d43804cf4dbf3c1cef0")
        reaper.Main_OnCommand(commandID2, 0) -- Custom: ReaTrak Set to Intro Post Fill
    end,
    
    intro_fill = function()
        commandID4 = reaper.NamedCommandLookup("_90d9c3359514c547af6790ca8994a692")
        reaper.Main_OnCommand(commandID4, 0) -- Custom: ReaTrak Set to Intro Fill
    end,

    verse_post_fill = function()
        commandID5 = reaper.NamedCommandLookup("_9a584ba9b330d343a95c2aa93fa6117b")
        reaper.Main_OnCommand(commandID5, 0) -- Custom: ReaTrak Set to Verse Post Fill
    end,

    verse = function()
        commandID6 = reaper.NamedCommandLookup("_ee462747fde1024f9cbab032dc78f12a")
        reaper.Main_OnCommand(commandID6, 0) -- Custom: ReaTrak Set to Verse
    end,

    verse_fill = function()
        commandID7 = reaper.NamedCommandLookup("_705d2ad890997c47910db95a51711f24")
        reaper.Main_OnCommand(commandID7, 0) -- Custom: ReaTrak Set to Verse Fill
    end,
    
    verse_ending = function()
        commandID8 = reaper.NamedCommandLookup("_a8a68b8ec61b344493682802d9ac44a4")
        reaper.Main_OnCommand(commandID8, 0) -- Custom: ReaTrak Set to Ending Verse
    end, 
    
    set_drum_riff = function()
        commandID171 = reaper.NamedCommandLookup("_5e1fe04adc6fe64d92f8e19d2fb2f59f")
        reaper.Main_OnCommand(commandID171, 0) -- Custom: ReaTrak Set to Drum Riff
    end,    
    
    bridge_post_fill = function()
        commandID9 = reaper.NamedCommandLookup("_34073107cc237b4f98d4c3dffd500c90")
        reaper.Main_OnCommand(commandID9, 0) -- Custom: ReaTrak Set to Bridge Post Fill
    end,

    bridge = function()
        commandID10 = reaper.NamedCommandLookup("_41fcba1bbfc22647825c27de49981278")
        reaper.Main_OnCommand(commandID10, 0) -- Custom: ReaTrak Set to Bridge
    end,

    bridge_fill = function()
        commandID11 = reaper.NamedCommandLookup("_0760f57fdfe45641ba1da32b9e6155aa")
        reaper.Main_OnCommand(commandID11, 0) -- Custom: ReaTrak Set to Bridge Fill
    end,
    
    pre_chorus_post_fill = function()
        commandID12 = reaper.NamedCommandLookup("_715ac062b062dc4ba22ddc50252f1e76")
        reaper.Main_OnCommand(commandID12, 0) -- Custom: ReaTrak Set to Pre Chorus Post Fill
    end,

    pre_chorus = function()
        commandID13 = reaper.NamedCommandLookup("_cbf39ae594389349ba38f9a5b1175059")
        reaper.Main_OnCommand(commandID13, 0) -- Custom: ReaTrak Set to Pre Chorus
    end,

    pre_chorus_fill = function()
        commandID14 = reaper.NamedCommandLookup("_3fe2d48d4d0eba4eae0ddd0731d3410f")
        reaper.Main_OnCommand(commandID14, 0) -- Custom: ReaTrak Set to Pre Chorus Fill
    end,
    
    chorus_post_fill = function()
        commandID15 = reaper.NamedCommandLookup("_9f3494ace8dc654ca2f8731d56fba4c4")
        reaper.Main_OnCommand(commandID15, 0) -- Custom: ReaTrak Set to Chorus Post Fill
    end,

    chorus = function()
        commandID16 = reaper.NamedCommandLookup("_11153a391144ba4c959ddd4915885bfd")
        reaper.Main_OnCommand(commandID16, 0) -- Custom: ReaTrak Set to Chorus
    end,

    chorus_fill = function()
        commandID17 = reaper.NamedCommandLookup("_226c876dff5f82488ad78626dfce652b")
        reaper.Main_OnCommand(commandID17, 0) -- Custom: ReaTrak Set to Chorus Fill
    end,
    
    chorus_ending = function()
        commandID18 = reaper.NamedCommandLookup("_f2f3a75e68c966459bdf4fa0460389a8")
        reaper.Main_OnCommand(commandID18, 0) -- Custom: ReaTrak Set to Ending Chorus
    end,    
  
    rest = function()
        commandID22 = reaper.NamedCommandLookup("_c7b48c7e54f8e94f9f3437b58f81d549")
        reaper.Main_OnCommand(commandID22, 0) -- Custom: ReaTrak Set to Rest
    end,

    shot = function()
        commandID23 = reaper.NamedCommandLookup("_929e978567d96b439647f7437c867c86")
        reaper.Main_OnCommand(commandID23, 0) -- Custom: ReaTrak Set to Shot
    end,
    
    hold = function()
        commandID24 = reaper.NamedCommandLookup("_36088becd3ad3948b6d69148121ae590")
        reaper.Main_OnCommand(commandID24, 0) -- Custom: ReaTrak Set to Hold
    end,
    
    
}

local mnu_actions = {
    
    help = function()
        commandID30 = reaper.NamedCommandLookup("_RSc8cc178fa82743e63c0937889c5d21f018f89025")
        reaper.Main_OnCommand(commandID30, 0) -- Script: ReaTrak actions help.lua
    end,
    
    show_chords_in_items = function()
        reaper.Main_OnCommand(40691, 0) -- View: Toggle show media cues in items
    end, 
    
    arranger = function()
        commandID44 = reaper.NamedCommandLookup("_S&M_SHOW_RGN_PLAYLIST")
        reaper.Main_OnCommand(commandID44, 0) -- SWS/S&M: Open/close Region Playlist window
    end,       
    
    choose_instrument = function()
        commandID31 = reaper.NamedCommandLookup("_affc9a3267b5384da92c67a48aeb8998")
        reaper.Main_OnCommand(commandID31, 0) -- Custom: ReaTrak Choose Instrument
    end,    

    reatrak = function()
        commandID32 = reaper.NamedCommandLookup("_dc512c9369370044a2f8a292c27ca39d")
        reaper.Main_OnCommand(commandID32, 0) -- Custom: ReaTrak
    end,    

    reatrak_selection = function()
        commandID33 = reaper.NamedCommandLookup("_0aaf7120a177a044b41d609683934470")
        reaper.Main_OnCommand(commandID33, 0) -- Custom: ReaTrak Selection
    end,    

    redo_selection = function()
        commandID34 = reaper.NamedCommandLookup("_0aaf7120a177a044b41d609683934470")
        reaper.Main_OnCommand(commandID34, 0) -- Custom: ReaTrak Redo Selection
    end,    

    midi_natation = function()
        commandID35 = reaper.NamedCommandLookup("_243882a556b4a24390679255a508233c")
        reaper.Main_OnCommand(commandID35, 0) -- Custom: ReaTrak Get MIDI Notation for Audio from mid file
    end,    

    import_chords = function()
        reaper.Main_OnCommand(41759, 0) -- Markers/Regions: Import markers/regions from file (replace all existing)
    end,   

    midi_chord_track = function()
        commandID36 = reaper.NamedCommandLookup("_8f65d65ed51ecf4c947500e10cc72333")
        reaper.Main_OnCommand(commandID36, 0) -- Custom: ReaTrak Create MIDI Chords from Region Chord name
    end,   

    decay_fill_2bars = function()
        commandID37 = reaper.NamedCommandLookup("_RS5bc5e3f22e0523c58534ef3d3c38cee3a974262c")
        reaper.Main_OnCommand(commandID37, 0) -- Script: ReaTrak lengthen selected item by 2 bars.lua
    end,    
    
    musicxml_chords = function()
        commandID37 = reaper.NamedCommandLookup("_RS375dbf0f2902e41294d13434abfc48e46e54a560")
        reaper.Main_OnCommand(commandID37, 0) -- Script: ReaTrak musicxml chords notation events to regions.lua
    end,    

    chords_from_midi_markers = function()
        commandID38 = reaper.NamedCommandLookup("_6d6a7a1f3903ce468e1d09422f81ea08")
        reaper.Main_OnCommand(commandID38, 0) -- Custom: ReaTrak Import Chord Markers from MIDI Track to Regions
    end,    

    chords_from_midi_lyrics = function()
        commandID39 = reaper.NamedCommandLookup("_RS583f37fc5569edab89f5b140ae2a67f85f2f1218")
        reaper.Main_OnCommand(commandID39, 0) -- Script: ReaTrak add regions from midi lyrics selected items.lua
    end,    

    text_chord_track = function()
        commandID41 = reaper.NamedCommandLookup("_25a40d32e8a51e4490f20d848e204f44")
        reaper.Main_OnCommand(commandID41, 0) -- Custom: ReaTrak Track Text Chords
    end,

    biab_drums = function()
        commandID42 = reaper.NamedCommandLookup("_RS6c08e9df64c8bae1935a9ff8f2b237688090b305")
        reaper.Main_OnCommand(commandID42, 0) -- Script: ReaTrak biab drums.lua
    end,        

    snap_midi_to_chords = function()
        commandID43 = reaper.NamedCommandLookup("_RSe5a47e6b8b345105272f6eeec66bd1f1e1c43892")
        reaper.Main_OnCommand(commandID43, 0) -- Script: ReaTrak snap midi takes to chords.lua
    end,

    drum_riff_selection = function()
        commandID19 = reaper.NamedCommandLookup("_cd48802cf959d24899b5fd6375d9e202")
        reaper.Main_OnCommand(commandID19, 0) -- Custom: ReaTrak Drum Riff Selection
    end,

    redo_drum_riff = function()
        commandID20 = reaper.NamedCommandLookup("_94bf61335e356940ba7c7133e0157412")
        reaper.Main_OnCommand(commandID20, 0) -- Custom: ReaTrak Redo Drum Riff
    end,
    
    remove_drum_riff = function()
        commandID21 = reaper.NamedCommandLookup("_84f0ef8db34384428d97be197f4bf540")
        reaper.Main_OnCommand(commandID21, 0) -- Custom: ReaTrak Remove Drum Riff
    end,
    
    decay_all_D_on_track = function()
        commandID22 = reaper.NamedCommandLookup("_RSe58c776d6645bdbe9fd063dd48c2f8d699e11591")
        reaper.Main_OnCommand(commandID22, 0) -- Script: ReaTrak decay all -D items on track.lua
    end,

    circle_of_fifths = function()
        commandID23 = reaper.NamedCommandLookup("_RS5c33378c0bda081e673d359fd650ee9c540e531a")
        reaper.Main_OnCommand(commandID23, 0) -- Script: ReaTrak Circle of Fifths Menu.lua
    end, 
    
    reatrak_gui = function()
        commandID23 = reaper.NamedCommandLookup("_RSd2180a7db83ee51fd6c8932087ff504bbca97485")
        reaper.Main_OnCommand(commandID23, 0) -- Script: ReaTrak gui.lua
    end,     
    
    xml_to_abc = function()
        commandID24 = reaper.NamedCommandLookup("_RSa687019a2a93d65396400ca02330e54ae719765a")
        reaper.Main_OnCommand(commandID24, 0) -- Script: ReaTrak xml to abc.lua
    end,     

    chord_sheet = function()
        commandID25 = reaper.NamedCommandLookup("_RS4a4ed7141dc91d3fb42c0c955a3ae8489fc7e371")
        reaper.Main_OnCommand(commandID25, 0) -- Script: ReaTrak mpl_Region chord editor.lua
    end,     
}

local mnu_edit = { 
    
    help = function()
        commandID51 = reaper.NamedCommandLookup("_RS68073fe0041179348e0fb6f7d0f85e85f1b9c3a4")
        reaper.Main_OnCommand(commandID51, 0) -- Script: ReaTrak edit help.lua
    end,
    
    insert_chord = function()
        commandID52 = reaper.NamedCommandLookup("_4190cb5a012a9a4db06a253f79f13a85")
        reaper.Main_OnCommand(commandID52, 0) -- Custom: ReaTrak Insert Chord
    end,

    insert_chord_selection = function()
        reaper.Main_OnCommand(40306, 0) -- Markers: Insert region from time selection and edit
    end, 

    edit_chord = function()
        reaper.Main_OnCommand(40616, 0) -- Markers: Edit region near cursor
    end,    
      
    split_chord_region = function()
        commandID53 = reaper.NamedCommandLookup("_RS8fa94cd55cd82c3d9b0a066f7f5ceec2009c7180")
        reaper.Main_OnCommand(commandID53, 0) -- Script: ReaTrak Split region under cursor.eel
    end,

    chord_regions = function()
        reaper.Main_OnCommand(40326, 0) -- View: Show region/marker manager window
    end,    

    snap_chords_grid = function()
        commandID54 = reaper.NamedCommandLookup("_RS0b4230c08f384a4318c16aac4e2404c9fef86887")
        reaper.Main_OnCommand(commandID54, 0) -- Script: ReaTrak Snap all regions to grid.eel
    end,
  
    transpose_chords = function()
        commandID55 = reaper.NamedCommandLookup("_RS243506a4ece5cd8f1050d89c241baaee2b286a15")
        reaper.Main_OnCommand(commandID55, 0) -- Script: ReaTrak transpose chords selection.lua
    end,
  
    insert_bars = function()
        commandID56 = reaper.NamedCommandLookup("_cb76e351ba62ac4c8da62b1d7432f521")
        reaper.Main_OnCommand(commandID56, 0) -- Custom: ReaTrak Insert Bars
    end,
  
    remove_bars = function()
        commandID57 = reaper.NamedCommandLookup("_e7fa1c6be44c19409e56de09357c6889")
        reaper.Main_OnCommand(commandID57, 0) -- Custom: ReaTrak Remove Bars
    end,
  
    copy_chords = function()
        commandID58 = reaper.NamedCommandLookup("_RS86c331d8ad1119ad855aa6311d9116aa8d22a6dd")
        reaper.Main_OnCommand(commandID58, 0) -- Script: ReaTrak Copy project markers and regions in time selection.lua
    end,
  
    paste_chords = function()
        commandID59 = reaper.NamedCommandLookup("_28e11f451e11d3408dcc9339c74ea29f")
        reaper.Main_OnCommand(commandID59, 0) -- Custom: ReaTrak Paste Insert Regions Markers
    end,
  
    glue_track = function()
        commandID60 = reaper.NamedCommandLookup("_5e8c07d444b762489b550bdfd7f8ed39")
        reaper.Main_OnCommand(commandID60, 0) -- Custom: ReaTrak Glue Track Items
    end,

    export_chords = function()
        commandID61 = reaper.NamedCommandLookup("_456b491816ecba44932ec63ea5e3faed")
        reaper.Main_OnCommand(commandID61, 0) -- Custom: ReaTrak Export Chords
    end,    

    export_renumber_chords = function()
        commandID62 = reaper.NamedCommandLookup("_1d118cb7528a39459e792639d3501064")
        reaper.Main_OnCommand(commandID62, 0) -- Custom: ReaTrak Export/Renumber Chords
    end,
      
    decay_fill_2bars = function()
        commandID63 = reaper.NamedCommandLookup("_RS5bc5e3f22e0523c58534ef3d3c38cee3a974262c")
        reaper.Main_OnCommand(commandID63, 0) -- Script: ReaTrak lengthen selected item by 2 bars.lua
    end,
    
    remove_selected_crossfade = function()
        commandID64 = reaper.NamedCommandLookup("_eb97fc0933296c48a1ddfff48d6c3f22")
        reaper.Main_OnCommand(commandID64, 0) -- Custom: ReaTrak remove crossfade
    end, 
    
    remove_track_crossfades = function()
        commandID65 = reaper.NamedCommandLookup("_7bbc0eab6fc73c4b935e263650cbc3ce")
        reaper.Main_OnCommand(commandID65, 0) -- Custom: ReaTrak remove all crossfades from track
    end,

    selected_notes_to_chord_name = function()
        commandID66 = reaper.NamedCommandLookup("_RSc73f9a4ab45a0812b2d415c79ecf756f553b54ba")
        reaper.Main_OnCommand(commandID66, 0) -- Script: ReaTrak selected notes to chord name and reascale.lua
    end,          
    
    split_region_at_bars = function()
        commandID67 = reaper.NamedCommandLookup("_36b8e76821195744b46d4b5156f12182")
        reaper.Main_OnCommand(commandID67, 0) -- Custom: ReaTrak split region at bars
    end,
    
    split_region_at_beats = function()
        commandID68 = reaper.NamedCommandLookup("_b14cce354562434393ccddfa629158f8")
        reaper.Main_OnCommand(commandID68, 0) -- Custom: ReaTrak split region at beats
    end, 
    
    transpose_selected_items = function()
        commandID69 = reaper.NamedCommandLookup("_RSe421cdcd255a30db79468d6587813761466a9506")
        reaper.Main_OnCommand(commandID69, 0) -- Script: ReaTrak transpose selected items.lua
    end,  
    
}


local mnu_devl = {

    help = function()
        commandID70 = reaper.NamedCommandLookup("_RSbd3da65fa96aecedb5d4ec28fb293c8e73ed56fc")
        reaper.Main_OnCommand(commandID70, 0) -- Script: ReaTrak development help.lua
    end,

    set_drum_riff = function()
        commandID71 = reaper.NamedCommandLookup("_5e1fe04adc6fe64d92f8e19d2fb2f59f")
        reaper.Main_OnCommand(commandID71, 0) -- Custom: ReaTrak Set to Drum Riff
    end,    

    set_play_anywhere = function()
        commandID72 = reaper.NamedCommandLookup("_98f3c748ca7b214aa9a7a1f5644c2cd5")
        reaper.Main_OnCommand(commandID72, 0) -- Custom: ReaTrak Set to Play Anywhere
    end,
    
    set_to_decay1 = function()
        commandID73 = reaper.NamedCommandLookup("_a0450467fcb72a48a75c77cf0d3cd925")
        reaper.Main_OnCommand(commandID73, 0) -- Custom: ReaTrak set to decay 1 bar
    end,        
    
    set_to_decay2 = function()
        commandID76 = reaper.NamedCommandLookup("_323f3dd46b2d3f40b0f507e0046a4bb1")
        reaper.Main_OnCommand(commandID76, 0) -- Custom: ReaTrak set to decay
    end,       

    remove_set_to_decay = function()
        commandID74 = reaper.NamedCommandLookup("_aeead470bd831849b64fe54a572c935b")
        reaper.Main_OnCommand(commandID74, 0) -- Custom: ReaTrak remove set to decay
    end,    

    remove_linked = function()
        commandID75 = reaper.NamedCommandLookup("_e6b9cad8910e2a448e812ce62b961a5f")
        reaper.Main_OnCommand(commandID75, 0) -- Custom: ReaTrak remove linked
    end,    
    
}


------------------------------------
-------- Menu contents -------------
------------------------------------





-- This table is passed to the Menubar
-- Must be structured like this (.title, .options, etc)
local menus = {
    
    {title = "Sections", options = {
        {"Help",                 mnu_sections.help},
        {">Count-In"},                
        {"<Count-In",       mnu_sections.count_in},
        {">Intro"},
            {"Intro",          mnu_sections.intro},
            {"Intro Post Fill",           mnu_sections.intro_post_fill},
            {"<Intro Fill",           mnu_sections.intro_fill},
        {">Verse"},
             {"Verse",          mnu_sections.verse},
             {"Verse Post Fill",           mnu_sections.verse_post_fill},
             {"Verse Fill",           mnu_sections.verse_fill},
             {"<Verse Ending",           mnu_sections.verse_ending},
        {">Bridge"},
             {"Bridge",          mnu_sections.bridge},
             {"Bridge Post Fill",           mnu_sections.bridge_post_fill},
             {"<Bridge Fill",           mnu_sections.bridge_fill},
        {">Pre Chorus"},
             {"Pre Chorus",          mnu_sections.pre_chorus},
             {"Pre Chorus Post Fill",           mnu_sections.pre_chorus_post_fill},
             {"<Pre Chorus Fill",           mnu_sections.pre_chorus_fill},
        {">Chorus"},
             {"Chorus",          mnu_sections.chorus},
             {"Chorus Post Fill",           mnu_sections.chorus_post_fill},
             {"Chorus Fill",           mnu_sections.chorus_fill},
             {"<Chorus Fill Ending",           mnu_sections.chorus_ending}, 
        {">Drum Riff"},
             {"<Set Drum Riff",          mnu_sections.set_drum_riff},
        {">Rest Shot Hold"},
             {"Rest",          mnu_sections.rest},
             {"Shot",           mnu_sections.shot},
             {"<Hold",           mnu_sections.hold},                                                    
        {""},

    }},
 
    {title = "Actions", options = {
        {"Help",                 mnu_actions.help},
        {">ReaTrak"},
            {"Choose Instrument",  mnu_actions.choose_instrument},
            {"ReaTrak",        mnu_actions.reatrak},
            {"ReaTrak Selection",        mnu_actions.reatrak_selection},
            {"Redo Selection",        mnu_actions.redo_selection},
            {"Decay all -D on Track",        mnu_actions.decay_all_D_on_track},
            {"<MIDI Notation",  mnu_actions.midi_notation},
        {">Drum Riff"},
             {"Drum Riff Selection",           mnu_actions.drum_riff_selection},
             {"Redo Drum Riff",           mnu_actions.redo_drum_riff},
             {"<Remove Drum Riff",           mnu_actions.remove_drum_riff},        
        {">Chords"},
             {"Import Chords",          mnu_actions.import_chords},
             {"Chord Regions",          mnu_actions.show_regions},
             {"Export Renumber Chords",           mnu_actions.export_renumbers_chords},
             {"Export Chords",           mnu_actions.export_chords},
             {"Chords from MusicXML",           mnu_actions.chords_from_musicxml},
             {"XML to ABC",           mnu_actions.xml_to_abc},
             {"Chords from MIDI Markers",           mnu_actions.chords_from_midi_markers},
             {"Chords from MIDI Lyrics",           mnu_actions.chord_from_midi_lyrics},
             {"Snap Chords to Grid",           mnu_actions.snap_chords_to_grid},
             {"MIDI Chord Track",           mnu_actions.midi_chord_track},
             {"<Text Chord Track",           mnu_actions.text_chord_track},
        {">MIDI Snap"},
             {"Snap MIDI Item to Chords",           mnu_actions.snap_midi_items_to_chords},
             {"<Snap MIDI to Scale",           mnu_actions.snap_midi_to_scale},
        {"Circle of Fifths",                 mnu_actions.circle_of_fifths}, 
        {"ReaTrak GUI",                 mnu_actions.reatrak_gui}, 
        {"Chord Sheet",                 mnu_actions.chord_sheet},                           
     }},
    
    {title = "Edit", options = {
        {"Help",                 mnu_edit.help},
        {"Arranger",       mnu_edit.arranger},
        {">Copy"},
             {"<Copy Selected Chords",          mnu_edit.copy_chords},
        {">Paste"},
             {"<Paste Selected Chords at Cursor",          mnu_edit.paste_chords},  
        {">Split Region"},
             {"Split Region @ Cursor",          mnu_edit.split_chord_region},  
             {"Split Region @ Beats",          mnu_edit.split_region_at_beats},
             {"<Split Region @ Bars",          mnu_edit.split_region_at_bars},
        {">Glue"},                             
        {"<Glue Items on Track",  mnu_edit.toggle_full_screen},
        {">Transpose"},
             {"Transpose Selected Chords",          mnu_edit.transpose_selected_chords},  
             {"Transpose MIDI In Chord Region",          mnu_edit.transpose_midi_region},
             {"<Transpose Selected Items",          mnu_edit.transpose_selected_items},
        {">Crossfades"},
             {"Remove Selected Croosfade",          mnu_edit.remove_selected_crossfade},  
             {"<Remove Track Crossfades",          mnu_edit.remove_track_crossfades},
                    
    }},
    
    
    
    {title = "Developmant", options = {
        {"Help",                mnu_devl.help},
        {"Set Drum Riff",  mnu_devl.set_drum_riff},
        {"Set Play Anywhere",  mnu_devl.set_play_anywhere},        
        {">Linked"},
             {"Set Linked",          mnu_devl.set_linked},  
             {"<Remove Linked",          mnu_edit.remove_linked},         
        {">Decay"},
             {"Set Decay 1 bar",          mnu_devl.set_to_decay1},  
             {"Set Decay 2 bar",          mnu_edit.set_to_decay2},
             {"<Remove Set Decay",          mnu_edit.remove_set_to_decay},            
    }},
}











------------------------------------
-------- Window settings -----------
------------------------------------

GUI.name = "ReaTrak - Menubar"
GUI.x, GUI.y, GUI.w, GUI.h = 0, 0, 300, 40
GUI.anchor, GUI.corner = "mouse", "C"




------------------------------------
-------- GUI Elements --------------
------------------------------------


--[[
    
    Menubar     z,  x,  y,  menus[, w, h, pad])
  Button    z,   x,   y,   w,   h, caption, func[, ...]
  Listbox    z,   x,   y,   w,   h[, list, multi, caption, pad])
  TextEditor  z,  x,  y,  w,  h[, text, caption, pad])
  
]]--


GUI.New("mnu_menu", "Menubar",      1,  0,   0,   menus, GUI.w)
--GUI.New("lst_titles", "Listbox",  1,  16,  40,  300, 208, "", true)
--GUI.New("btn_go", "Button",      1,  324, 104, 32,  24, "-->", add_text) 
--GUI.New("txted_text", "TextEditor",  1,  364, 40,  420, 208, "Select an item\nor two\nor three\nor everything\n\nin the list and click the button!")


--GUI.elms.lst_titles.list = titles


--function GUI.elms.lst_titles:ondoubleclick()

--  add_text()
  
--end


GUI.Init()
GUI.Main()
