-- @description Search selected notes in chord finder
-- @version 1.1
-- @author Mordi
-- @changelog
--  This script will take any selected notes and generate a URL
--  that will show what chord the notes make, if any.
--  It uses the wonderful site "www.scales-chords.com" to
--  do this.
--
--  Made by Mordi, Jan 2016

-- Modified fo Local Chord database

-----DEBUG CONTROL--------------------------------------------------------------

debug_output = true -- true/false --display debug messages in the console

-----DISPLAY CONSOLE MESSAGE----------------------------------------------------

-- Display a message in the console for debugging, message can be a number variable
-- or a string but not mixed. No line feed character is added.
function Msg(value)
  --returns nothing

  if debug_output then --check if flag is set true (at the start of this script)
    reaper.ShowConsoleMsg(tostring(value) .. "\n" )
  end
end

-- Function for opening a URL
function OpenURL(url)
  local OS = reaper.GetOS()
  if OS == "OSX32" or OS == "OSX64" then
    os.execute('open "" "' .. url .. '"')
  else
    os.execute('start "" "' .. url .. '"')
  end
end

-- Init note name-array (for use in URL)
noteNames = {"C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B"}

-- Get HWND
hwnd = reaper.MIDIEditor_GetActive()
if not hwnd then
    Msg("Chord finder: Whoops! No MIDI editor found.")
    return
end

-- Get current take being edited in MIDI Editor
take = reaper.MIDIEditor_GetTake(hwnd)
if not take then
    Msg("Chord finder: Whoops! No MIDI take found.")
    return
end

-- Check if the take exists
if take == nil then
  Msg("Chord finder: Whoops! No take selected.")
else
  -- Count all notes in take
  retval, notes, ccs, sysex = reaper.MIDI_CountEvts(take)

  -- Setup variables for number of selected notes
  -- and an array to hold the pitch of selected notes
  -- selNum = 0 -- CHANGE: we can just use the length of the note table ( #selNotePitch )
  selNotePitch = {}
  
  -- Loop through every note in take, and get selected notes
  -- and their pitch
  for i = 0, notes + 1 do
    retval, sel, muted, startppqposOut, endppqposOut, chan, pitch, vel = reaper.MIDI_GetNote(take, i)
    
    if sel == true then
      -- Don't add note if it already exists
      note_is_dupe = false
      for n = 1, #selNotePitch do -- CHANGE: the For loops need to start from 1 for # to work
        if (selNotePitch[n] % 12) == (pitch % 12) then
          note_is_dupe = true
        end
      end
      if not note_is_dupe then -- Add note to array
        selNotePitch[#selNotePitch + 1] = pitch  -- % 12 -- CHANGE: keep the original note number for now
       
        -- Msg(pitch) - CHANGE: we need to sort the values first
      end
       
    end
  end

  Msg("Found " .. #selNotePitch .. " notes\n")

  -- CHANGE: sort the table
  table.sort(selNotePitch)
  
  

  
  -- If more than 7 notes are selected, we can't generate a URL
  if (#selNotePitch > 7) then
    Msg("Chord-finder: Whoops! Can't select more than 7 notes.")
  else
    -- If no notes are selected we can't generate a URL
    if (#selNotePitch == 0) then
      Msg("Chord-finder: Whoops! No notes selected.")
    else
      -- Generate chord notes
      foundnotes = ""
      notenum1 = "" 
      for i = 1, #selNotePitch do
        --foundnotes = foundnotes .. "," .. noteNames[selNotePitch[i] % 12 + 1] .. "" -- CHANGE: Get the single-octave note value here instead
        foundnotes = foundnotes .. "," .. noteNames[selNotePitch[i] % 12 + 1] .. "" -- CHANGE: Get the single-octave note value here instead
        
        
        -- CHANGE: we're looping through the table here anyway, so we can Msg the notes at the same time
        Msg(i .. ":\t" .. selNotePitch[i])
        notenum1 = notenum1 .. "," .. selNotePitch[i] - selNotePitch[1] .. ""
        
      end
    end

end 

     
notenumbers = ""     
  local map = {A = 9, ["A#"] = 10, B = 11, C = 0, ["C#"] = 1, D = 2, ["D#"] = 3, E = 4, F = 5, ["F#"] = 6, G = 7, ["G#"] = 8}
  local base, prev
  chordnotes = (foundnotes:gsub("%,", "\1",1):gsub("\1","", 1))
  notenums = (notenum1:gsub("%,", "\1",1):gsub("\1","", 1)) 
  notenums5 = (chordnotes:gsub("([^,]+)", function(note) --Not used
  

        local num = map[note]
        base = base or num
        num = num - base
        if prev and num < prev then num = num + 12 end
        prev = num
        return tostring(num)
      end))
    end

root = string.sub(chordnotes, 1, 2)
chordroot = (root:gsub("%,", "\1",1):gsub("\1","", 1))


--Msg("\nRoot:\t" .. root)
Msg("\nRoot:\t" .. chordroot)
   
Msg("Notes:\t" .. chordnotes)
Msg("Numbers:\t" .. notenums)



if notenums == "0,4,7"             then chordtype = "Maj"     end
if notenums == "0,3,7"             then chordtype = "m"       end
if notenums == "0,3,6"             then chordtype = "dim"     end
if notenums == "0,4,8"             then chordtype = "aug"     end
if notenums == "0,4,6"             then chordtype = "-5"      end
if notenums == "0,2,7"             then chordtype = "sus2"    end
if notenums == "0,5,7"             then chordtype = "sus4"    end
if notenums == "0,7"               then chordtype = "5"       end
if notenums == "0,3,8"             then chordtype = "m#5"     end
if notenums == "0,7,10"            then chordtype = "5add7"   end
if notenums == "0,2,4,7"           then chordtype = "add2"    end
if notenums == "0,4,5,7"           then chordtype = "add4"    end
if notenums == "0,3,5,7"           then chordtype = "madd4"   end
if notenums == "0,4,7,9"           then chordtype = "6"       end
if notenums == "0,3,7,9"           then chordtype = "m6"      end
if notenums == "0,4,5,7,9"         then chordtype = "6add4"   end
if notenums == "0,3,5,7,9"         then chordtype = "m6add4"  end
if notenums == "0,4,7,9,14"        then chordtype = "6add9"   end
if notenums == "0,3,7,9,14"        then chordtype = "m6add9"  end
if notenums == "0,2,7,9"           then chordtype = "6sus2"   end
if notenums == "0,5,7,9"           then chordtype = "6sus4"   end
if notenums == "0,4,7,9,17"        then chordtype = "6add11"  end
if notenums == "0,3,7,9,17"        then chordtype = "m6add11" end
if notenums == "0,4,7,10"          then chordtype = "7"       end
if notenums == "0,2,4,7,10"        then chordtype = "7add2"   end
if notenums == "0,4,5,7,10"        then chordtype = "7add4"   end
if notenums == "0,3,7,10"          then chordtype = "m7"      end
if notenums == "0,3,5,7,10"        then chordtype = "m7add4"  end
if notenums == "0,4,7,11"          then chordtype = "Maj7"    end
if notenums == "0,3,6,9"           then chordtype = "dim7"    end
if notenums == "0,3,7,11"          then chordtype = "mMaj7"   end
if notenums == "0,3,7,11,17"       then chordtype = "mMaj7add11"   end
if notenums == "0,3,7,11,21"       then chordtype = "mMaj7add13"   end
if notenums == "0,2,7,10"          then chordtype = "7sus2"   end
if notenums == "0,5,7,10"          then chordtype = "7sus4"   end
if notenums == "0,5,6,10"          then chordtype = "7susb5"  end
if notenums == "0,5,6,10,13"       then chordtype = "7susb5b9"    end
if notenums == "0,5,6,10,13,20"    then chordtype = "7susb5b9b13" end
if notenums == "0,5,6,10,14,20"    then chordtype = "7susb5b13"   end
if notenums == "0,5,6,10,15"       then chordtype = "7susb5#9"    end
if notenums == "0,5,6,10,15,20"    then chordtype = "7susb5#9b13" end
if notenums == "0,5,7,10,13"       then chordtype = "7susb9"  end
if notenums == "0,5,7,10,13,20"    then chordtype = "7susb9b13"   end
if notenums == "0,5,7,10,13,18"    then chordtype = "7susb9#11"   end
if notenums == "0,5,7,10,13,18,20" then chordtype = "7susb9#11b13"  end
if notenums == "0,5,7,10,20"       then chordtype = "7susb13" end
if notenums == "0,5,8,10"          then chordtype = "7sus#5"  end
if notenums == "0,5,8,10,15,18"    then chordtype = "7sus#5#9#11" end
if notenums == "0,5,8,10,18"       then chordtype = "7sus#5#11"   end
if notenums == "0,5,7,10,15"       then chordtype = "7sus#9"  end
if notenums == "0,5,7,10,15,20"    then chordtype = "7sus#9b13"   end
if notenums == "0,5,7,10,15,18,20" then chordtype = "7sus#9#11b13"  end
if notenums == "0,5,7,10,18"       then chordtype = "7sus#11" end
if notenums == "0,5,7,10,18,20"    then chordtype = "7sus#11b13"  end
if notenums == "0,2,7,11"          then chordtype = "Maj7sus2"    end
if notenums == "0,5,7,11"          then chordtype = "Maj7sus4"    end
if notenums == "0,4,8,10"          then chordtype = "aug7"    end
if notenums == "0,4,6,10"          then chordtype = "7b5"     end
if notenums == "0,4,6,10,20"       then chordtype = "7b5b13"  end
if notenums == "0,4,6,10,15,20"    then chordtype = "7b5#9b13"    end
if notenums == "0,4,8,10"          then chordtype = "7#5"     end
if notenums == "0,4,8,10,18"       then chordtype = "7#5#11"  end
if notenums == "0,4,8,10,13,18"    then chordtype = "7#5b9#11"    end
if notenums == "0,4,8,10,15,18"    then chordtype = "7#5#9#11"    end
if notenums == "0,4,7,10,15,18,20" then chordtype = "7#9#11b13"   end
if notenums == "0,4,7,10,14,18,20" then chordtype = "7#11b13" end
if notenums == "0,3,6,10"          then chordtype = "m7b5"    end
if notenums == "0,3,8,10"          then chordtype = "m7#5"    end
if notenums == "0,4,6,11"          then chordtype = "Maj7b5"  end
if notenums == "0,4,8,11"          then chordtype = "Maj7#5"  end
if notenums == "0,4,7,10,13"       then chordtype = "7b9"     end
if notenums == "0,4,7,10,15"       then chordtype = "7#9"     end
if notenums == "0,3,7,10,13"       then chordtype = "m7b9"    end
if notenums == "0,3,7,10,15"       then chordtype = "m7#9"    end
if notenums == "0,4,7,11,13"       then chordtype = "Maj7b9"  end
if notenums == "0,4,7,11,15"       then chordtype = "Maj7#9"  end
if notenums == "0,4,7,10,13,20"    then chordtype = "7b9b13"  end
if notenums == "0,3,7,10,13,20"    then chordtype = "m7b9b13" end
if notenums == "0,4,7,10,14,20"    then chordtype = "7b13"    end
if notenums == "0,3,7,10,14,20"    then chordtype = "m7b13"   end
if notenums == "0,4,7,10,15,20"    then chordtype = "7#9b13"  end
if notenums == "0,3,7,10,15,20"    then chordtype = "m7#9b13" end
if notenums == "0,4,6,10,13"       then chordtype = "7b5b9"   end
if notenums == "0,4,6,10,15"       then chordtype = "7b5#9"   end
if notenums == "0,4,8,10,13"       then chordtype = "7#5b9"   end
if notenums == "0,4,8,10,15"       then chordtype = "7#5#9"   end
if notenums == "0,4,7,10,18"       then chordtype = "7#11"    end
if notenums == "0,4,7,9,10"        then chordtype = "7add6"   end
if notenums == "0,4,7,10,17"       then chordtype = "7add11"  end
if notenums == "0,4,7,10,21"       then chordtype = "7add13"  end
if notenums == "0,3,7,10,17"       then chordtype = "m7add11" end
if notenums == "0,3,6,10,13"       then chordtype = "m7b5b9"  end
if notenums == "0,3,6,10,15"       then chordtype = "m7b5#9"  end
if notenums == "0,3,8,10,13"       then chordtype = "m7#5b9"  end
if notenums == "0,3,8,10,15"       then chordtype = "m7#5#9"  end
if notenums == "0,3,7,10,18"       then chordtype = "m7#11"   end
if notenums == "0,4,6,11,13"       then chordtype = "Maj7b5b9"    end
if notenums == "0,4,6,11,15"       then chordtype = "Maj7b5#9"    end
if notenums == "0,4,8,11,13"       then chordtype = "Maj7#5b9"    end
if notenums == "0,4,8,11,15"       then chordtype = "Maj7#5#9"    end
if notenums == "0,4,7,11,17"       then chordtype = "Maj7add11"   end
if notenums == "0,4,7,11,18"       then chordtype = "Maj7#11" end
if notenums == "0,4,7,10,14"       then chordtype = "9"       end
if notenums == "0,3,7,10,14"       then chordtype = "m9"      end
if notenums == "0,4,7,11,14"       then chordtype = "Maj9"    end
if notenums == "0,5,7,11,14"       then chordtype = "Maj9sus4"    end
if notenums == "0,3,7,11,14"       then chordtype = "mMaj9"   end
if notenums == "0,5,7,10,14"       then chordtype = "9sus4"   end
if notenums == "0,5,6,10,14,17"    then chordtype = "9susb5"  end
if notenums == "0,5,6,10,14"       then chordtype = "9susb5"  end
if notenums == "0,5,8,10,14"       then chordtype = "9sus#5"  end
if notenums == "0,5,6,10,14,17,20" then chordtype = "9sus5b13"    end
if notenums == "0,5,7,10,14,18"    then chordtype = "9sus#11" end
if notenums == "0,5,8,10,14,17"    then chordtype = "9susb5#9"    end
if notenums == "0,5,8,10,14,18"    then chordtype = "9sus#5#11"   end
if notenums == "0,4,7,10,15"       then chordtype = "aug9"    end
if notenums == "0,4,7,9,10,14"     then chordtype = "9add6"   end
if notenums == "0,3,7,9,14"        then chordtype = "m9add6"  end
if notenums == "0,4,6,10,14"       then chordtype = "9b5"     end
if notenums == "0,4,6,10,14,20"    then chordtype = "9b5b13"  end
if notenums == "0,4,7,10,14,20"    then chordtype = "9b13"    end
if notenums == "0,4,6,10,14,20"    then chordtype = "9b5b13"  end
if notenums == "0,4,8,10,14,18"    then chordtype = "9#5#11"  end
if notenums == "0,4,8,10,14"       then chordtype = "9#5"     end
if notenums == "0,4,7,10,14,18"    then chordtype = "9#11"    end
if notenums == "0,4,7,10,14,18,20" then chordtype = "9#11b13" end
if notenums == "0,3,6,10,14"       then chordtype = "m9b5"    end
if notenums == "0,3,8,10,14"       then chordtype = "m9#5"    end
if notenums == "0,4,6,11,14"       then chordtype = "Maj9b5"  end
if notenums == "0,4,8,11,14"       then chordtype = "Maj9#5"  end
if notenums == "0,4,7,11,14,18"    then chordtype = "Maj9#11" end
if notenums == "0,4,7,10,13,18"    then chordtype = "b9#11"   end
if notenums == "0,4,7,14"          then chordtype = "add9"    end
if notenums == "0,3,7,14"          then chordtype = "madd9"   end
if notenums == "0,4,7,17"          then chordtype = "add11"   end
if notenums == "0,3,7,17"          then chordtype = "madd11"  end
if notenums == "0,4,7,10,14,17"    then chordtype = "11"      end
if notenums == "0,5,7,10,14,17"    then chordtype = "11sus4"  end
if notenums == "0,3,7,10,14,17"    then chordtype = "m11"     end
if notenums == "0,4,7,11,14,17"    then chordtype = "Maj11"   end
if notenums == "0,3,7,11,14,17"    then chordtype = "mMaj11"  end
if notenums == "0,4,7,10,14,18"    then chordtype = "aug11"   end
if notenums == "0,3,7,10,14,18"    then chordtype = "augm11"  end
if notenums == "0,4,6,10,14,17"    then chordtype = "11b5"    end
if notenums == "0,4,8,10,14,17"    then chordtype = "11#5"    end
if notenums == "0,4,7,10,13,17"    then chordtype = "11b9"    end
if notenums == "0,4,7,10,15,17"    then chordtype = "11#9"    end
if notenums == "0,4,6,10,13,17"    then chordtype = "11b5b9"  end
if notenums == "0,4,8,10,13,17"    then chordtype = "11#5b9"  end
if notenums == "0,4,6,10,15,17"    then chordtype = "11b5#9"  end
if notenums == "0,4,8,10,15,17"    then chordtype = "11#5#9"  end
if notenums == "0,3,6,10,14,17"    then chordtype = "m11b5"   end
if notenums == "0,3,8,10,14,17"    then chordtype = "m11#5"   end
if notenums == "0,3,7,10,13,17"    then chordtype = "m11b9"   end
if notenums == "0,3,7,10,15,17"    then chordtype = "m11#9"   end
if notenums == "0,3,6,10,13,17"    then chordtype = "m11b5b9" end
if notenums == "0,3,8,10,13,17"    then chordtype = "m11#5b9" end
if notenums == "0,3,6,10,15,17"    then chordtype = "m11b5#9" end
if notenums == "0,3,8,10,15,17"    then chordtype = "m11#5#9" end
if notenums == "0,4,6,11,14,17"    then chordtype = "Maj11b5" end
if notenums == "0,4,8,11,14,17"    then chordtype = "Maj11#5" end
if notenums == "0,4,7,11,13,17"    then chordtype = "Maj11b9" end
if notenums == "0,4,7,11,15,17"    then chordtype = "Maj11#9" end
if notenums == "0,4,6,11,13,17"    then chordtype = "Maj11b5b9"   end
if notenums == "0,4,8,11,13,17"    then chordtype = "Maj11#5b9"   end
if notenums == "0,4,6,11,15,17"    then chordtype = "Maj11b5#9"   end
if notenums == "0,4,8,11,15,17"    then chordtype = "Maj11#5#9"   end
if notenums == "0,4,7,10,14,21"    then chordtype = "13"      end
if notenums == "0,4,7,10,14,17,21" then chordtype = "13"      end
if notenums == "0,3,7,10,14,17,21" then chordtype = "m13"     end
if notenums == "0,4,7,11,14,17,21" then chordtype = "Maj13"   end
if notenums == "0,4,7,11,14,21"    then chordtype = "Maj13"   end
if notenums == "0,4,8,10,14,21"    then chordtype = "13aug"   end
if notenums == "0,3,7,11,14,17,21" then chordtype = "mMaj13"  end
if notenums == "0,4,6,10,14,17,21" then chordtype = "13b5"    end
if notenums == "0,4,6,10,21"       then chordtype = "13b5"    end
if notenums == "0,4,6,9,10"        then chordtype = "13b5"    end
if notenums == "0,4,8,9,10"        then chordtype = "13#5"    end
if notenums == "0,4,8,10,21"       then chordtype = "13#5"    end
if notenums == "0,4,8,10,14,17,21" then chordtype = "13#5"    end
if notenums == "0,4,7,10,13,21"    then chordtype = "13b9"    end
if notenums == "0,4,7,10,13,17,21" then chordtype = "13b9"    end
if notenums == "0,4,7,10,15,17,21" then chordtype = "13#9"    end
if notenums == "0,4,7,10,15,21"    then chordtype = "13#9"    end
if notenums == "0,4,6,10,13,21"    then chordtype = "13b5b9"  end
if notenums == "0,4,6,10,13,17,21" then chordtype = "13b5b9"  end
if notenums == "0,4,8,10,13,21"    then chordtype = "13#5b9"  end
if notenums == "0,4,8,10,13,17,21" then chordtype = "13#5b9"  end
if notenums == "0,4,6,10,15,21"    then chordtype = "13b5#9"  end
if notenums == "0,4,6,10,15,17,21" then chordtype = "13b5#9"  end
if notenums == "0,4,8,10,15,17,21" then chordtype = "13#5#9"  end
if notenums == "0,4,8,10,13,18,21" then chordtype = "13#5b9#11"   end
if notenums == "0,4,8,10,15,18,21" then chordtype = "13#5#9#11"   end
if notenums == "0,4,7,10,13,18,21" then chordtype = "13b9#11" end
if notenums == "0,4,7,10,15,18,21" then chordtype = "13#9#11" end
if notenums == "0,4,7,10,14,18,21" then chordtype = "13#11"   end
if notenums == "0,3,7,10,14,18,21" then chordtype = "m13#11"  end
if notenums == "0,5,7,10,21"       then chordtype = "13sus4"  end
if notenums == "0,7,10,17,21"      then chordtype = "13sus4"  end
if notenums == "0,7,10,17,21"      then chordtype = "13sus4"  end
if notenums == "0,5,6,10,14,21"    then chordtype = "13susb5" end
if notenums == "0,5,6,10,13,21"    then chordtype = "13susb5b9"   end
if notenums == "0,5,6,10,15,21"    then chordtype = "13susb5#9"   end
if notenums == "0,5,7,10,13,21"    then chordtype = "13susb9" end
if notenums == "0,5,7,10,13,18,21" then chordtype = "13susb9#11"  end
if notenums == "0,5,8,10,21"       then chordtype = "13sus#5" end
if notenums == "0,5,8,10,13,18,21" then chordtype = "13sus#5b9#11"  end
if notenums == "0,5,8,10,16,21"    then chordtype = "13sus#5#11"  end
if notenums == "0,5,8,10,15,18,21" then chordtype = "13sus#5#9#11"  end
if notenums == "0,5,7,10,15,21"    then chordtype = "13sus#9" end
if notenums == "0,5,7,10,15,18,21" then chordtype = "13sus#9#11"  end
if notenums == "0,5,7,10,14,18,21" then chordtype = "13sus#11"    end
if notenums == "0,3,6,10,14,17,21" then chordtype = "m13b5"   end
if notenums == "0,3,8,10,14,17,21" then chordtype = "m13#5"   end
if notenums == "0,3,7,10,13,17,21" then chordtype = "m13b9"   end
if notenums == "0,3,7,10,15,17,21" then chordtype = "m13#9"   end
if notenums == "0,3,6,10,13,17,21" then chordtype = "m13b5b9" end
if notenums == "0,3,8,10,13,17,21" then chordtype = "m13#5b9" end
if notenums == "0,3,6,10,15,17,21" then chordtype = "m13b5#9" end
if notenums == "0,3,8,10,15,17,21" then chordtype = "m13#5#9" end
if notenums == "0,4,6,11,14,17,21" then chordtype = "Maj13b5" end
if notenums == "0,4,8,11,14,17,21" then chordtype = "Maj13#5" end
if notenums == "0,4,7,11,13,17,21" then chordtype = "Maj13b9" end
if notenums == "0,4,7,11,15,17,21" then chordtype = "Maj13#9" end
if notenums == "0,4,6,11,13,17,21" then chordtype = "Maj13b5b9"   end
if notenums == "0,4,8,11,13,17,21" then chordtype = "Maj13#5b9"   end
if notenums == "0,4,6,11,15,17,21" then chordtype = "Maj13b5#9"   end
if notenums == "0,4,8,11,15,17,21" then chordtype = "Maj13#5#9"   end
if notenums == "0,4,7,11,14,18,21" then chordtype = "Maj13#11"    end








if not chordtype then Msg("Chord not found") 
notenumbers = ""
return
end
--chord = ("Chord:\t" .. chordroot .. "" .. chordtype)
chord = (chordroot .. "" .. chordtype)

Msg("Chord:\t" .. chord)

--Convert Chord to Reascale

x1=1
x2=0
x3=0
x4=0
x5=0
x6=0
x7=0
x8=0
x9=0
x10=0
x11=0
x12=0




--Cnotes = {0, 4, 7, 10}

local str = (notenums)
local t = {}
for num in string.gmatch(str, "%d+") do
    t[#t+1] = tonumber(num)
end

--xxxxx CHANGED 8 then x9=5
Cnotes = t

  if Cnotes[2] == 1 then x2=2
  elseif Cnotes[2] == 2 then x3=2
  elseif Cnotes[2] == 3 then x4=3
  elseif Cnotes[2] == 4 then x5=3
  elseif Cnotes[2] == 5 then x6=4
  elseif Cnotes[2] == 6 then x7=5
  elseif Cnotes[2] == 7 then x8=5
  elseif Cnotes[2] == 8 then x9=5
  elseif Cnotes[2] == 9 then x10=6
  elseif Cnotes[2] == 10 then x11=7
  elseif Cnotes[2] == 11 then x12=7
  elseif Cnotes[2] == 13 then x2=9
  elseif Cnotes[2] == 14 then x3=9
  elseif Cnotes[2] == 15 then x4=9
  elseif Cnotes[2] == 16 then x5=9
  elseif Cnotes[2] == 17 then x6="B"
  elseif Cnotes[2] == 18 then x7="B"

  end
  
  if Cnotes[3] == 3 then x4=3 
  elseif Cnotes[3] == 4 then x5=3
  elseif Cnotes[3] == 5 then x6=4
  elseif Cnotes[3] == 6 then x7=5
  elseif Cnotes[3] == 7 then x8=5
  elseif Cnotes[3] == 8 then x9=5  
  elseif Cnotes[3] == 9 then x10=6
  elseif Cnotes[3] == 10 then x11=7
  elseif Cnotes[3] == 11 then x12=7
  elseif Cnotes[3] == 13 then x2=9
  elseif Cnotes[3] == 14 then x3=9
  elseif Cnotes[3] == 15 then x4=9
  elseif Cnotes[3] == 16 then x5=9
  elseif Cnotes[3] == 17 then x6="B"
  elseif Cnotes[3] == 18 then x7="B"

  end
  
  if Cnotes[4] == 5 then x6=4
  elseif Cnotes[4] == 6 then x7=5
  elseif Cnotes[4] == 7 then x8=5
  elseif Cnotes[4] == 8 then x9=5
  elseif Cnotes[4] == 9 then x10=6
  elseif Cnotes[4] == 10 then x11=7
  elseif Cnotes[4] == 11 then x12=7
  elseif Cnotes[4] == 13 then x2=9
  elseif Cnotes[4] == 14 then x3=9
  elseif Cnotes[4] == 15 then x4=9
  elseif Cnotes[4] == 16 then x5=9
  elseif Cnotes[4] == 17 then x6="B"
  elseif Cnotes[4] == 18 then x7="B"  
  elseif Cnotes[4] == 19 then x8="B"
  elseif Cnotes[4] == 20 then x9="B"
  elseif Cnotes[4] == 21 then x10="D"
  elseif Cnotes[4] == 22 then x11="D"
  elseif Cnotes[4] == 23 then x12="D"
   
  end  


  if Cnotes[5] == 6 then x7=5 
  elseif Cnotes[5] == 7 then x8=5
  elseif Cnotes[5] == 8 then x9=5
  elseif Cnotes[5] == 9 then x10=6
  elseif Cnotes[5] == 10 then x11=7
  elseif Cnotes[5] == 11 then x12=7
  elseif Cnotes[5] == 13 then x2=9
  elseif Cnotes[5] == 14 then x3=9
  elseif Cnotes[5] == 15 then x4=9
  elseif Cnotes[5] == 16 then x5=9
  elseif Cnotes[5] == 17 then x6="B"
  elseif Cnotes[5] == 18 then x7="B"
  elseif Cnotes[5] == 19 then x8="B"
  elseif Cnotes[5] == 20 then x9="B"
  elseif Cnotes[5] == 21 then x10="D"
  elseif Cnotes[5] == 22 then x11="D"
  elseif Cnotes[5] == 23 then x12="D"
  
  end  


  
  if Cnotes[6] == 7 then x8=6
  elseif Cnotes[6] == 8 then x9=6
  elseif Cnotes[6] == 9 then x10=6
  elseif Cnotes[6] == 10 then x11=7
  elseif Cnotes[6] == 11 then x12=7
  elseif Cnotes[6] == 13 then x2=9
  elseif Cnotes[6] == 14 then x3=9
  elseif Cnotes[6] == 15 then x4=9
  elseif Cnotes[6] == 16 then x5=9
  elseif Cnotes[6] == 17 then x6="B"
  elseif Cnotes[6] == 18 then x7="B"
  elseif Cnotes[6] == 19 then x8="B"
  elseif Cnotes[6] == 20 then x9="B"
  elseif Cnotes[6] == 21 then x10="D"
  elseif Cnotes[6] == 22 then x11="D"
  elseif Cnotes[6] == 23 then x12="D"
  
  end   


  if Cnotes[7] == 7 then x8=7
  elseif Cnotes[7] == 8 then x9=7
  elseif Cnotes[7] == 9 then x10=7
  elseif Cnotes[7] == 10 then x11=7
  elseif Cnotes[7] == 11 then x12=7
  elseif Cnotes[7] == 13 then x2=9
  elseif Cnotes[7] == 14 then x3=9
  elseif Cnotes[7] == 15 then x4=9
  elseif Cnotes[7] == 16 then x5=9
  elseif Cnotes[7] == 17 then x6="B"
  elseif Cnotes[7] == 18 then x7="B"
  elseif Cnotes[7] == 19 then x8="B"
  elseif Cnotes[7] == 20 then x9="B"
  elseif Cnotes[7] == 21 then x10="D"
  elseif Cnotes[7] == 22 then x11="D"
  elseif Cnotes[7] == 23 then x12="D"
  
  end   


reascale = (x1 .. x2 .. x3 .. x4 .. x5 .. x6 .. x7 .. x8 .. x9 .. x10 .. x11 .. x12)




              

if Cnotes[3] == nil then Cnotes[3] = 0 end
if Cnotes[4] == nil then Cnotes[4] = 0 end
if Cnotes[5] == nil then Cnotes[5] = 0 end
if Cnotes[6] == nil then Cnotes[6] = 0 end
if Cnotes[7] == nil then Cnotes[7] = 0 end


Msg("Reascale:" .. reascale)
